import {
    createStore as createReduxStore,
    applyMiddleware,
    compose,
    combineReducers,
} from 'redux'
import thunk from 'redux-thunk'

import { routerMiddleware } from 'react-router-redux'
import { ReduxEvents } from 'redux-events-middleware'
import { createSSRContext } from './create-ssr-context'

import {
    getReducers as getFeaturesReducers,
    decorateStore as decorateStoreWithFeatures,
} from 'react-redux-feature/lib/decorate-store'

export const createAppStateFactory = (appReducers = {}, appFeatures = []) => async (initialState = {}, history = null) => {
    const events = new ReduxEvents()
    const ssrContext = createSSRContext(initialState.ssr || {})

    const middlewares = [
        thunk,
        routerMiddleware(history),
        events.createReduxMiddleware({ history }),
    ]

    // @DEV: redux dev tools (development & client only)
    const enhancers = []
    if (process.env.NODE_ENV === 'development' && !process.env.SSR) {
        const { devToolsExtension } = window

        if (typeof devToolsExtension === 'function') {
            enhancers.push(devToolsExtension())
        }
    }

    const composedEnhancers = compose(
        applyMiddleware(...middlewares),
        ...enhancers,
    )

    const initialReducers = {
        ...appReducers,
        ...getFeaturesReducers(appFeatures),
        ...ssrContext.reducers,
    }

    const combinedReducers = combineReducers(initialReducers)

    // redux store gets created
    let store = createReduxStore(
        combinedReducers,
        initialState,
        composedEnhancers,
    )

    // react-redux-features
    // add features capabilities to the store
    store = decorateStoreWithFeatures({ store, history, events, initialReducers })

    // Initialize dynamic stuff
    await store.registerSyncFeatures(appFeatures)
    await store.startSyncFeatures()

    // @DEV: expose the store for the console to dispatch actions
    if (process.env.NODE_ENV === 'development' && !process.env.SSR) {
        window.store = store
    }

    return {
        store,
        history,
        events,
        ssrContext,
    }
}
